@@ -243,11 +243,16 @@ esd_connect_unix(void)
   if ( socket_out < 0 )
     return -1;
   /* this was borrowed blindly from the Tcl socket stuff */
-  if ( fcntl( socket_out, F_SETFD, FD_CLOEXEC ) < 0 )
+  if ( fcntl( socket_out, F_SETFD, FD_CLOEXEC ) < 0 ) {
+    close (socket_out);
     return -1;
+  }
   if ( setsockopt( socket_out, SOL_SOCKET, SO_REUSEADDR,
-		  &curstate, sizeof(curstate) ) < 0 )
+		  &curstate, sizeof(curstate) ) < 0 ) {
+    close (socket_out);
     return -1;
+  }
+
   /* set the connect information */
   socket_unix.sun_family = AF_UNIX;
   strncpy(socket_unix.sun_path, ESD_UNIX_SOCKET_NAME, sizeof(socket_unix.sun_path));
@@ -443,7 +448,6 @@ int open_listen_socket(const char *hostname, int port )
         {
 		if (!(resolved=gethostbyname2(hostname, AF_INET)))
                 {
-			hstrerror(h_errno);
 			return -1;
                 }
                 memcpy(&(socket_addr.sin_addr), resolved->h_addr_list[0], resolved->h_length);
@@ -616,6 +620,9 @@ int main ( int argc, char *argv[] )
     char *hostname=NULL;
     char *endptr;
 
+    int fd0;
+    int max_fds;
+
     /* from esd_config.c */
     extern char esd_spawn_options[];
     extern char esd_default_options[];
@@ -737,7 +744,7 @@ int main ( int argc, char *argv[] )
 	} else if ( !strcmp( opts[ arg ], "-as" ) ) {
 	    if ( ++arg != num_opts ) {
 		esd_autostandby_secs = strtol ( opts[arg], &endptr, 10);
-		if ( !esd_autostandby_secs && !endptr && !*endptr) {
+		if ( !esd_autostandby_secs && (!endptr || !*endptr) ) {
 		    esd_autostandby_secs = ESD_DEFAULT_AUTOSTANDBY_SECS;
 		    fprintf( stderr, "- could not read autostandby timeout: %s\n",
 			     opts[ arg ] );
@@ -816,6 +823,15 @@ int main ( int argc, char *argv[] )
 	}
     }
 
+    /* close all open file descriptors */
+    max_fds = getdtablesize();
+    for(i=0;i<max_fds;i++){
+      close(i);
+    }
+    fd0 = open("/dev/null", O_RDWR);
+    dup(0);
+    dup(0);
+
     /* open the listening socket */
   listen_socket = open_listen_socket(hostname, esd_port );
   if ( listen_socket < 0 ) {
@@ -1004,7 +1020,8 @@ int main ( int argc, char *argv[] )
 	get_new_clients( listen_socket );
 
 
-	if ((esd_clients_list == NULL) && (!first) && (esd_terminate)) {
+	if ((esd_clients_list == NULL) && (!esd_playing_samples) &&
+            (!first) && (esd_terminate)) {
 /*	  fprintf(stderr, "No clients!\n");*/
 	  clean_exit(0);
 	  exit(0);
@@ -1093,6 +1110,8 @@ int main ( int argc, char *argv[] )
 	}
     } /* while ( 1 ) */
 
+    close (listen_socket);
+
     /* how we'd get here, i have no idea, should only exit on signal */
     clean_exit( -1 );
     exit( 0 );
