@@ -26,8 +26,6 @@
 
 #if defined(__GNUC__) && !defined(__STRICT_ANSI__)
 
-#define _GNU_SOURCE 1
-
 #ifdef DSP_DEBUG
 #define DPRINTF(format, args...)	printf(format, ## args)
 #else
@@ -46,6 +44,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <stdio.h>
+#include <errno.h>
 
 #ifdef HAVE_MACHINE_SOUNDCARD_H
 #  include <machine/soundcard.h>
@@ -72,13 +71,10 @@ typedef unsigned long request_t;
 typedef int request_t;
 #endif
 
-static int sndfd = -1;
-static int mixfd = -1;
-static int settings = 0;
-static int done = 0;
+static int sndfd = -1, mixfd = -1;
+static int settings = 0, done = 0;
 
-static char *ident = NULL;
-static char *mixer = NULL;
+static char *ident = NULL, *mixer = NULL;
 static int use_mixer = 0;
 
 /*
@@ -136,9 +132,8 @@ dsp_init (void)
       const char *str;
      
       str = getenv ("ESDDSP_NAME");
-      ident = malloc (ESD_NAME_MAX+1);
+      ident = malloc (ESD_NAME_MAX);
       strncpy (ident, (str ? str : "esddsp"), ESD_NAME_MAX);
-      ident[ESD_NAME_MAX] = '\0';
 
       str = getenv("ESDDSP_MMAP");
       mmapemu = str && !strcmp(str,"1");
@@ -220,29 +215,21 @@ static void mmapemu_flush(void)
     }
 }
 
-typedef int (*open_funcptr_t) (const char *, int, mode_t);
-
-/* The wrap_* functions act as wrapper for both the 32-bit versions of
- * the functions (open/mmap/fopen), as well as for the 64-bit ones
- * (open64/mmap64/fopen64). Just to avoid massive code duplication.
- *   -- Sebastian Schmidt <yath@yath.de>, 2005-12-15
- */
-int
-wrap_open (const char *symname, open_funcptr_t *func, const char *pathname,
-                int flags, mode_t mode)
+static int
+open_wrapper (int (*func) (const char *, int, mode_t),
+	      const char *pathname, int flags, ...)
 {
-  if (!*func) {
-    *func = (open_funcptr_t) dlsym (REAL_LIBC, symname);
-    dsp_init ();
-  }
+  va_list args;
+  mode_t mode;
+
+  dsp_init ();
+
+  va_start (args, flags);
+  mode = va_arg (args, mode_t);
+  va_end (args);
 
   if (!strcmp (pathname, "/dev/dsp"))
     {
-/* We shouldn't need this with Unix domain sockets, plus there
- * is a better test in esdlib.c - for more info see:
- * http://www.debian.org/Bugs/db/53/53602.html
- * commented out by bma@debian.org 1999-12-29 */
-/*
       if (!getenv ("ESPEAKER") && !mmapemu)
 	{
           int ret;
@@ -251,54 +238,149 @@ wrap_open (const char *symname, open_funcptr_t *func, const char *pathname,
 	  if ((ret = (*func) (pathname, flags, mode)) >= 0)
 	    return ret;
 	}
-*/
 
-      DPRINTF ("hijacking /dev/dsp %s, and taking it to esd...\n", symname);
+      DPRINTF ("hijacking /dev/dsp open, and taking it to esd...\n");
       settings = done = 0;
       return (sndfd = esd_open_sound (NULL));
     }
   else if (use_mixer && !strcmp (pathname, "/dev/mixer"))
     {
-      DPRINTF ("hijacking /dev/mixer %s, and taking it to esd...\n", symname);
-      return (mixfd = (**func) (mixer, O_RDWR, S_IRUSR | S_IWUSR));
+      DPRINTF ("hijacking /dev/mixer open, and taking it to esd...\n");
+      return (mixfd = (*func) (mixer, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR));
     }
   else
-    return (**func) (pathname, flags, mode);
+    return (*func) (pathname, flags, mode);
 }
 
 int
 open (const char *pathname, int flags, ...)
 {
-  static open_funcptr_t func = NULL;
-  mode_t mode = 0;
+  static int (*func) (const char *, int, mode_t) = NULL;
   va_list args;
-  
-  if (flags & O_CREAT) {
-    va_start(args, flags);
-    mode = va_arg(args, mode_t);
-    va_end(args);
-  }
+  mode_t mode;
 
-  return wrap_open("open", &func, pathname, flags, mode);
+  DPRINTF ("open\n");
+
+  if (!func) 
+    {
+      func = (int (*) (const char *, int, mode_t)) dlsym (REAL_LIBC, "open");
+      if (!func) 
+	{
+	  fprintf(stderr, "esddsp: error: Cannot find symbol 'open'\n");
+	  errno = ENXIO;
+	  return -1;
+	}
+    }
+
+  va_start (args, flags);
+  mode = va_arg (args, mode_t);
+  va_end (args);
+
+  return open_wrapper(func, pathname, flags, mode);
 }
 
-#ifdef HAVE_OPEN64
 int
 open64 (const char *pathname, int flags, ...)
 {
-  static open_funcptr_t func = NULL;
-  mode_t mode = 0;
+  static int (*func) (const char *, int, mode_t) = NULL;
   va_list args;
-  
-  if (flags & O_CREAT) {
-    va_start(args, flags);
-    mode = va_arg(args, mode_t);
-    va_end(args);
-  }
+  mode_t mode;
 
-  return wrap_open("open64", &func, pathname, flags, mode);
+  DPRINTF ("open64\n");
+
+  if (!func) 
+    {
+      func = (int (*) (const char *, int, mode_t)) dlsym (REAL_LIBC, "open64");
+      if (!func) 
+	{
+	  fprintf(stderr, "esddsp: error: Cannot find symbol 'open64'\n");
+	  errno = ENXIO;
+	  return -1;
+	}
+    }
+
+  va_start (args, flags);
+  mode = va_arg (args, mode_t);
+  va_end (args);
+
+  return open_wrapper(func, pathname, flags, mode);
+}
+
+static FILE *
+fopen_wrapper (FILE * (*func) (const char *, const char *),
+	       const char *pathname, const char *mode)
+{
+  dsp_init ();
+
+  if (!strcmp (pathname, "/dev/dsp"))
+    {
+      if (!getenv ("ESPEAKER") && !mmapemu)
+	{
+          FILE *ret;
+
+	  if ((ret = (*func) (pathname, mode)) != NULL)
+	    return ret;
+	}
+
+      DPRINTF ("hijacking /dev/dsp fopen, and taking it to esd...\n");
+      settings = done = 0;
+      return fdopen((sndfd = esd_open_sound (NULL)), mode);
+    }
+  else if (use_mixer && !strcmp (pathname, "/dev/mixer"))
+    {
+      FILE *ret;
+      DPRINTF ("hijacking /dev/mixer fopen, and taking it to esd...\n");
+
+      if ((ret = (*func) (pathname, mode)) != NULL)
+	{
+	  mixfd = fileno(ret);
+	  return ret;
+	}
+    }
+  else
+    return (*func) (pathname, mode);
+  /* FIXME: warning */
+}
+
+FILE *
+fopen (const char *path, const char *mode)
+{
+  static FILE * (*func) (const char *, const char *) = NULL;
+
+  DPRINTF ("fopen\n");
+
+  if (!func) 
+    {
+      func = (FILE * (*) (const char *, const char *)) dlsym (REAL_LIBC, "fopen");
+      if (!func) 
+	{
+	  fprintf(stderr, "esddsp: error: Cannot find symbol 'fopen'\n");
+	  return NULL;
+	}
+    }
+
+  return fopen_wrapper(func, path, mode);
+}
+
+FILE *
+fopen64 (const char *path, const char *mode)
+{
+  static FILE * (*func) (const char *, const char *) = NULL;
+
+  DPRINTF ("fopen64\n");
+
+  if (!func) 
+    {
+      func = (FILE * (*) (const char *, const char *)) dlsym (REAL_LIBC, "fopen64");
+      if (!func) 
+	{
+	  fprintf(stderr, "esddsp: error: Cannot find symbol 'fopen64'\n");
+	  return NULL;
+	}
+    }
+
+  return fopen_wrapper(func, path, mode);
 }
-#endif
 
 static int
 dspctl (int fd, request_t request, void *argp)
@@ -407,13 +489,13 @@ dspctl (int fd, request_t request, void *argp)
 
       done = 1;
 
-      if (write (fd, &proto, sizeof (proto)) != sizeof (proto))
+      if (write (sndfd, &proto, sizeof (proto)) != sizeof (proto))
         return -1;
-      if (write (fd, &fmt, sizeof (fmt)) != sizeof (fmt))
+      if (write (sndfd, &fmt, sizeof (fmt)) != sizeof (fmt))
         return -1;
-      if (write (fd, &speed, sizeof (speed)) != sizeof (speed))
+      if (write (sndfd, &speed, sizeof (speed)) != sizeof (speed))
         return -1;
-      if (write (fd, ident, ESD_NAME_MAX) != ESD_NAME_MAX)
+      if (write (sndfd, ident, ESD_NAME_MAX) != ESD_NAME_MAX)
         return -1;
 
       if (mmapemu)
@@ -444,7 +526,7 @@ dspctl (int fd, request_t request, void *argp)
   return 0;
 }
 
-static int
+int
 mixctl (int fd, request_t request, void *argp)
 {
   static int esd = -1, player = -1;
@@ -558,21 +640,20 @@ close (int fd)
   return (*func) (fd);
 }
 
-typedef void *(*mmap_funcptr_t) (void *, size_t, int, int, int, off_t);
-
 void *
-wrap_mmap (const char *symname, mmap_funcptr_t *func, void *start, 
-                size_t length, int prot, int flags, int fd, off_t offset)
+mmap (void  *start,  size_t length, int prot, int flags, int fd, off_t offset)
 {
-  if (!*func)
-    *func = (mmap_funcptr_t) dlsym (REAL_LIBC, symname);
+  static void *(*func) (void *, size_t, int, int, int, off_t) = NULL;
+  
+  if (!func)
+    func = (void * (*) (void *, size_t, int, int, int, off_t)) dlsym (REAL_LIBC, "mmap");
 
   if(fd != sndfd || sndfd == -1)
-    return (**func)(start,length,prot,flags,fd,offset);
+    return (*func)(start,length,prot,flags,fd,offset);
   else
     {
-      DPRINTF ("esddsp: %s - start = %x, length = %d, prot = %d\n",
-	       symname, start, length, prot);
+      DPRINTF ("esddsp: mmap - start = %x, length = %d, prot = %d\n",
+	       start, length, prot);
       DPRINTF ("      flags = %d, fd = %d, offset = %d\n",flags, fd,offset);
       if(mmapemu)
 	{
@@ -581,27 +662,11 @@ wrap_mmap (const char *symname, mmap_funcptr_t *func, void *start,
 	  mmapemu_ocount.ptr = mmapemu_ocount.blocks = mmapemu_ocount.bytes = 0;
 	  return mmapemu_obuffer;
 	}
-      else DPRINTF ("esddsp: /dev/dsp %s (unsupported, try -m option)...\n", symname);
+      else DPRINTF ("esddsp: /dev/dsp mmap (unsupported, try -m option)...\n");
     }
   return (void *)-1;
 }
 
-void *
-mmap (void *start, size_t length, int prot, int flags, int fd, off_t offset)
-{
-  static mmap_funcptr_t func = NULL;
-  return wrap_mmap("mmap", &func, start, length, prot, flags, fd, offset);
-}
-
-#ifdef HAVE_MMAP64
-void *
-mmap64 (void *start, size_t length, int prot, int flags, int fd, off_t offset)
-{
-  static mmap_funcptr_t func = NULL;
-  return wrap_mmap("mmap64", &func, start, length, prot, flags, fd, offset);
-}
-#endif
-
 int
 munmap (void *start, size_t length)
 {
@@ -697,54 +762,6 @@ connect (int fd, struct sockaddr *addr, int len)
 
 #endif /* MULTIPLE_X11AMP */
 
-#if ESDDSP_STDIO_EMU
-/*
- * use #include rather than linking, because we want to keep everything
- * static inside esddsp to be sure that we don't override application symbols
- */
-#include "stdioemu.c"
-
-typdef FILE *(*fopen_funcptr_t) (const char *, const char *);
-
-FILE *
-wrap_fopen (const char *symname, fopen_funcptr_t *func, const char *pathname,
-                const char *mode)
-{
-  static FILE *(*func) (const char *, const char *) = NULL;
-
-  if (!*func) {
-    *func = (fopen_funcptr_t) dlsym (REAL_LIBC, symname);
-    dsp_init ();
-  }
-
-  if (!strcmp (pathname, "/dev/dsp"))
-    {
-      return fake_fopen(pathname,mode);
-    }
-  else if (use_mixer && !strcmp (pathname, "/dev/mixer"))
-    {
-      return fake_fopen(pathname,mode);
-    }
-  else
-    return (**func) (pathname, mode);
-}
-FILE *
-fopen(const char *pathname, const char *mode)
-{
-  static fopen_funcptr_t func;
-  return wrap_fopen("fopen", &func, pathname, mode);
-}
-
-#ifdef HAVE_FOPEN64
-FILE *
-fopen64(const char *pathname, const char *mode)
-{
-  static fopen_funcptr_t func;
-  return wrap_fopen("fopen64", &func, pathname, mode);
-}
-#endif
-#endif
-
 #else /* __GNUC__ */
 
 void
@@ -754,3 +771,4 @@ nogcc (void)
 }
 
 #endif
+
